### TL;DR

Use this seed project to jump start your React-based project to build library component.
This seed project includes the following technology stack: React, ES6, SASS, Webpack,
Lazy Loading component supports, Type Checking with Babel Type Check, ESLint, Jest and Enzyme for Unit testing.

Step 1: Clone this repo
```
git clone http://htran@rite.sd.spawar.navy.mil/bitbucket/scm/dcgsnufs/ufs-react-lib-seed.git
cd ufs-react-lib-seed
```

Step 2: Install

```
npm install
```

Step 3: Start

```
npm run start
```

Step 4: Linting & Test

```
npm run lint
npm run test
```

Step 5: Watch

```
npm run watch
```

Step 5: Build
```
npm run build
Package the app to /lib folder for publish to NPM repository.
```


### Features / Benefits

Features

* React 16
* ES6
* ImmutableJS
* PreCSS ( supports SASS-like markup in your CSS )
* PostCSS ( it support CSS modules, and we recommended B.E.M style )
* Webpack 3
* Lazy Loading component supports
* Type Checking with Babel Type Check ( Flow syntax )
* ESLint for syntax check
* Jest and Enzyme for Unit testing

Workflow

* Development
  * Hot Module Reload during development
  * Built-in lightweight config system
  * Built-in fancy cli dashboard for reporting run time compile status
  * Built-in support for multiple device concurrent debugging
* Build
  * Produced a /lib folder that is ready to publish to NPM
* Productivity
  * Highly configurable build and workflow system ( webpack )
  * Minimal setup time and allow you to invest into things that matters
  * Everything automatic, you just care about development, nothing else \o/ Yeah ?!

If you are interested, please read the `package.json` for all installed modules and plugins.

## Table of Contents

Basic
1. [Installation](#installation)
1. [Initialize your project](#initialize-your-project)
1. [Suggested Workflow](#suggested-workflow)
1. [Folder Structure](#folder-structure)
1. [Production Readiness](#production-readiness)
1. [Configuration](#configuration)
1. [Port Configuration](#port-configuration)
1. [Installing Dependencies](#installing-dependencies)

Advanced
1. [Lazy Loading Component](#lazy-loading-component)
1. [Writing Unit Test](#writing-unit-test)
1. [Configure git commit hook](#configure-git-commit-hook)
1. [Multiple Device Concurrent Debugging](#multiple-device-concurrent-debugging)
1. [Developing Template](#developing-template)
1. [Production Optimization and Bundle Analysis](#production-optimization-and-bundle-analysis)
1. [Integration Note](#integration-note)
1. [QA](#qa)

Other
1. [Knowledge Base Reading](#knowledge-base-reading)
1. [How to Contribute](#how-to-contribute)
1. [Updates](#updates)

# Basic

## Installation


### Prerequisite

You need to have Node.js installed.

## Initialize your project

Now run the following commands in your terminal

**NOTE: You only need to run this once!**

```sh
$ npm install # This will install the necessary packages to use the app
```

**That's it!**


### To run the app in Development Mode

```sh
$ npm run start
```

Wait about 30 seconds for your development environment to initialize.

When it finishes, it automatically open your browser and go to `http://localhost:8090/`

If you will see the page of all the library components.


### List of NPM Commands


```sh
$ npm run start     # build and load the app using local server
$ npm run build     # build the /lib folder containing everything ready for publish to NPM repository.
$ npm run lint      # linting using ESLint
$ npm run test      # run test using Jest
$ npm run clean     # it runs before each build, so you don't need to
$ npm run watch     # Monitor for file changes and run lint + unit tests + document generation.
```


## Suggested Workflow


After you check out the repo, I will usually do the following :

0. Go to your project root in your host machine
1. Run `npm run start` in your terminal ( wait until the dashboard show complete status )
2. Your browser will automatically load the app at `localhost:8090`
3. Run 'npm run watch' to monitor for code change and run lint, test and document generation.
3. Make code changes
4. If there are compilation errors, you will see it in the terminal dashboard
5. Watch your code changes reflect on browser without refreshing
6. Repeat your development steps

## Production Readiness

When you are ready to publish the React library component to NPM repo.

1. Run `npm run build` and wait until it is done
2. Go to the project `lib`, you will see a index.js, package.json and the various React library components.
3. cd to /lib and run "npm publish" to publish to NPM repo.

### Configuring application

If you look at folder `config`, there are 3 files

`default.json` - all default configuration

## Port Configuration
There will be a chance that you will need your port to be other than `8090`.  For example, your local backend service might already take `8090`; Or you want to run multiple project, one on `8090` and one on `80901`.

If you are running one project that needs a different port, you can just modify one place
1) `default.json` --> `port` section.

But if you want to run multiple projects at the same time, you will need to configure ports in two places
1) `default.json` --> `port`
2) Dashboard port --> `package.json` --> `dev`  ( default dashboard port is `9901` )


## Writing Unit Test

We are using Jest and Enzyme for unit testing, please refer to the Knowledge Base section below for more information.

Please put your unit test in the same folder as the componnent/class your testing.  All unit test my end with .spec.js.

For example, if you have the component Hello.jsx in /src/components/hello/, then create the unit test Hello.spec.js in the same folder.

## Configure git commit hook

We are using `husky` for preventing developers check in bad commits. Please go to package.json and look for `husky` the default settings. `husky` supports any git hooks and you can configure it to fit your needs.


## Developing Template

The `dist/index.html` is a generated artifact. If look at our sample template at `src/assets/template/_default.html`, the `dist/index.html` is generated from that file.

## Production Optimization and Bundle Analysis

For optimization, you can tweak the `config` under the following.  This optimization covers basic use case, feel free to make modification to fit your needs.

```
  "optimization": {
    "commonMinCount": 3,   // how many times a module is used before we consider it a common module ?
    "cssExclusion": true,  // do you want to exclude css from this optimization ?
    ...
  },
```

This boilerplate shipped with `Production Bundle Analysis` tool which is `turn on` by default and will give you insight to optimize the production assets.

For example, when you run `npm run build` with `"analyzeMode": true`, you should see something like the following at the end of your build.

The tool is to help you understand what your bundle looks like and what is included in your bundle.

![Production Bundle Analysis](https://www.dropbox.com/s/zun4n4tgp059neh/bundle-analysis.png?raw=1)

You can turn `on` and `off` the analysis feature

```
  "optimization": {
    ...
    "analyzeMode": true,   // changing to false will turn it off
    "analyze": {
      "port": 8888  // it will open localhost:8888 and show your bundle analysis
    }
  },
```
