/*------------------------------------------------------------------------------
 * Classification:      UNCLASSIFIED
 * Organization:        SPAWAR Systems Center
 *                      53560 Hull St, San Diego, California 92152-5001, U.S.A
 *                      Property of the U.S. Government
 *
 * Description: This script copies the necessary files to /lib folder for publishing to NPM repo.
 *----------------------------------------------------------------------------*/
var path = require("path");
var fse = require("fs-extra");

const files = [
    "README.md",
    "LICENSE"
];

Promise.all(
    files.map((file) => copyFile(file))
)
    .then(() => createPackageFile());

function copyFile(file) {
    const libPath = resolveBuildPath(file);
    return new Promise((resolve) => {
        fse.copy(
            file,
            libPath,
            (err) => {
                if (err) throw err;
                resolve();
            }
        );
    })
        .then(() => console.log(`Copied ${file} to ${libPath}`));
}

function resolveBuildPath(file) {
    return path.resolve(__dirname, "../lib/", path.basename(file));
}

function createPackageFile() {
    return new Promise((resolve) => {
        fse.readFile(path.resolve(__dirname, "../package.json"), "utf8", (err, data) => {
            if (err) {
                throw err;
            }
            
            resolve(data);
        });
    })
        .then((data) => JSON.parse(data))
        .then((packageData) => {
            const {
                author,
                version,
                description,
                keywords,
                repository,
                license,
                bugs,
                homepage,
                peerDependencies,
                dependencies,
            } = packageData;
            
            const minimalPackage = {
                name: "@dcgsn/ufs-react-components",
                author,
                version,
                description,
                main: "./index.js",
                keywords,
                repository,
                license,
                bugs,
                homepage,
                peerDependencies,
                dependencies
            };
            
            return new Promise((resolve) => {
                const libPath = path.resolve(__dirname, "../lib/package.json");
                const data = JSON.stringify(minimalPackage, null, 2);
                fse.writeFile(libPath, data, (err) => {
                    if (err) throw (err);
                    console.log(`Created package.json in ${libPath}`);
                    resolve();
                });
            });
        });
}
